'use strict';

var sass = require('sass');
var rgbHex = require('rgb-hex');
var shortHexColor = require('shorten-css-hex');
var isPlainObject = require('is-plain-obj');
var parseColor = require('parse-color');
var parseUnit = require('parse-css-dimension');
var immutable = require('immutable');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var sass__default = /*#__PURE__*/_interopDefaultLegacy(sass);
var rgbHex__default = /*#__PURE__*/_interopDefaultLegacy(rgbHex);
var shortHexColor__default = /*#__PURE__*/_interopDefaultLegacy(shortHexColor);
var isPlainObject__default = /*#__PURE__*/_interopDefaultLegacy(isPlainObject);
var parseColor__default = /*#__PURE__*/_interopDefaultLegacy(parseColor);
var parseUnit__default = /*#__PURE__*/_interopDefaultLegacy(parseUnit);

/**
 * @typedef {import('../index').JsonValue} JsonValue
 * @typedef {import('../index').JsonObject} JsonObject
 * @typedef {import('../index').JsonArray} JsonArray
 */

/**
 * @param {sass.Value|undefined} value
 */
function getJsonValueFromSassValue(value) {
  let resolvedValue;
  if (value instanceof sass__default["default"].SassList) {
    resolvedValue = listToArray(value);
  } else if (value instanceof sass__default["default"].SassMap) {
    resolvedValue = mapToObject(value);
  } else if (value instanceof sass__default["default"].SassColor) {
    /** @type {[number, number, number]} */
    const rgbValue = [value.red, value.green, value.blue];
    const alphaValue = value.alpha;
    if (alphaValue === 1) {
      resolvedValue = shortHexColor__default["default"](`#${rgbHex__default["default"].apply(null, rgbValue)}`);
    } else {
      resolvedValue = `rgba(${rgbValue.join(',')},${alphaValue})`;
    }
  } else if (value instanceof sass__default["default"].SassNumber) {
    if (value.hasUnits) {
      resolvedValue = String(value.value) + value.numeratorUnits.last();
    } else {
      resolvedValue = Number(value.value);
    }
  } else if (value instanceof sass__default["default"].SassString) {
    resolvedValue = String(value.text);
  } else if (value instanceof sass__default["default"].SassBoolean) {
    resolvedValue = Boolean(value.value);
  } else {
    resolvedValue = null;
  }
  return resolvedValue;
}

/**
 * @param {sass.SassList} list
 */
function listToArray(list) {
  const length = list.asList.size;
  /** @type {JsonArray} */
  const data = [];
  for (const index of Array.from({
    length
  }).keys()) {
    const value = getJsonValueFromSassValue(list.get(index));
    data.push(value);
  }
  return data;
}

/**
 * @param {sass.SassMap} map
 */
function mapToObject(map) {
  const length = map.contents.size;
  /** @type {JsonObject} */
  const data = {};
  for (const index of Array.from({
    length
  }).keys()) {
    const resolvedValue = map.get(index);
    if (typeof resolvedValue !== 'undefined') {
      const key = String(resolvedValue.get(0));
      const value = getJsonValueFromSassValue(resolvedValue.get(1));
      data[key] = value;
    }
  }
  return data;
}

/**
 * @typedef {import('../index').JsonValue} JsonValue
 * @typedef {import('../index').JsonObject} JsonObject
 * @typedef {import('../index').JsonArray} JsonArray
 */

const unitTypes = ['length', 'angle', 'resolution', 'frequency', 'time'];

/**
 * @param {string} value
 */
function isColor(value) {
  return typeof parseColor__default["default"](value).rgba !== 'undefined';
}

/**
 * @param {string} value
 */
function parseValueToStringOrNumber(value) {
  let resolvedValue;
  try {
    const {
      value: parsedValue,
      unit,
      type
    } = parseUnit__default["default"](value);
    if (unitTypes.includes(type)) {
      resolvedValue = new sass__default["default"].SassNumber(parsedValue, unit);
    } else if (type === 'percentage') {
      resolvedValue = new sass__default["default"].SassNumber(parsedValue, '%');
    } else {
      resolvedValue = new sass__default["default"].SassString(value);
    }
  } catch (error) {
    resolvedValue = new sass__default["default"].SassString(value);
  }
  return resolvedValue;
}

/**
 * @param {string} value
 */
function parseValueToColor(value) {
  const [red, green, blue, alpha] = parseColor__default["default"](value).rgba;
  return new sass__default["default"].SassColor({
    red,
    green,
    blue,
    alpha
  });
}

/**
 * @param {JsonValue} value
 */
function setJsonValueToSassValue(value) {
  let resolvedValue;
  if (Array.isArray(value)) {
    resolvedValue = arrayToList(value);
  } else if (isPlainObject__default["default"](value)) {
    resolvedValue = objectToMap(value);
  } else if (isColor(String(value))) {
    resolvedValue = parseValueToColor(String(value));
  } else if (typeof value === 'string') {
    resolvedValue = parseValueToStringOrNumber(value);
  } else if (typeof value === 'number') {
    resolvedValue = new sass__default["default"].SassNumber(value);
  } else if (typeof value === 'boolean') {
    resolvedValue = value ? sass__default["default"].sassTrue : sass__default["default"].sassFalse;
  } else {
    resolvedValue = sass__default["default"].sassNull;
  }
  return resolvedValue;
}

/**
 * @param {JsonArray} array
 */
function arrayToList(array) {
  /** @type {sass.Value[]} */
  const data = [];
  for (const item of array) {
    data.push(setJsonValueToSassValue(item));
  }
  return new sass__default["default"].SassList(data);
}

/**
 * @param {JsonObject} object
 */
function objectToMap(object) {
  /** @type {[sass.Value, sass.Value][]} */
  const data = [];
  for (const [property, value = null] of Object.entries(object)) {
    data.push([setJsonValueToSassValue(property), setJsonValueToSassValue(value)]);
  }
  // eslint-disable-next-line new-cap
  return new sass__default["default"].SassMap(immutable.OrderedMap(data));
}

/**
 * @typedef {JsonPrimitive | JsonObject | JsonArray} JsonValue
 * @typedef {JsonValue[]} JsonArray
 * @typedef {string | number | boolean | null} JsonPrimitive
 * @typedef {{[Key in string]?: JsonValue}} JsonObject
 */

/**
 * Encodes (`JSON.stringify`) data and returns Sass string. By default, string is quoted with single quotes so that it can be easily used in standard CSS values.
 *
 * First argument:  `sass.Value` - Data to encode (stringify).
 *
 * Second argument: `sass.SassBoolean` - Should output string be quoted with single quotes.
 *
 * @param {sass.Value[]} encodeArguments
 */
function encode(encodeArguments) {
  const [data, quotes_] = encodeArguments;
  const quotes = quotes_.assertBoolean('quotes');
  const shouldQuote = quotes.value;
  let resolvedValue = JSON.stringify(getJsonValueFromSassValue(data));
  if (shouldQuote) {
    resolvedValue = `'${resolvedValue}'`;
  }
  return new sass__default["default"].SassString(resolvedValue);
}

/**
 * Decodes (`JSON.parse`) string and returns one of available Sass types.
 *
 * First argument: `sass.SassString` - String to decode (parse).
 *
 * @param {sass.Value[]} decodeArguments
 */
function decode(decodeArguments) {
  const [string_] = decodeArguments;
  const string = string_.assertString('string');
  /** @type {JsonValue?} */
  let resolvedValue = {};
  try {
    resolvedValue = JSON.parse(string.text);
  } catch (error) {
    resolvedValue = null;
  }
  return setJsonValueToSassValue(resolvedValue);
}

/** @type {{ 'json-encode($data, $quotes: true)': typeof encode, 'json-decode($string)': typeof decode }} */
const api = {
  'json-encode($data, $quotes: true)': encode,
  'json-decode($string)': decode
};

module.exports = api;
//# sourceMappingURL=index.js.map
